package com.gxzc.zen.web.sys.controller

import com.gxzc.zen.api.sys.model.SysRole
import com.gxzc.zen.api.sys.model.SysUser
import com.gxzc.zen.api.sys.service.ISysMenuService
import com.gxzc.zen.api.sys.service.ISysUserService
import com.gxzc.zen.common.base.BaseController
import com.gxzc.zen.common.config.response.annotation.ZenResponseFilter
import com.gxzc.zen.common.config.response.annotation.ZenResponseFilters
import com.gxzc.zen.common.dto.ResponseDto
import com.gxzc.zen.common.util.PaginationUtil
import com.gxzc.zen.umps.util.SSOUtil
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*

/**
 * 用户相关控制器
 * @author NorthLan
 * @date 2018/3/2
 * @url https://noahlan.com
 */
@RestController
@RequestMapping("user")
class UserController : BaseController() {
    companion object {
        private val logger = LoggerFactory.getLogger(UserController::class.java)
    }

    @Autowired
    private lateinit var userService: ISysUserService

    @Autowired
    private lateinit var menuService: ISysMenuService

    @ApiOperation("获取用户信息")
    @GetMapping("{id}")
    @ZenResponseFilter(type = SysUser::class, filter = ["createTime", "createBy", "updateTime", "updateBy", "password", "salt"])
    fun getById(@PathVariable id: Long): ResponseEntity<*> {
        return ResponseEntity.ok(ResponseDto().apply {
            data = userService.selectById(id)
        })
    }

    @ApiOperation("修改用户信息")
    @PutMapping
    @ZenResponseFilter(type = SysUser::class, filter = ["createTime", "createBy", "updateTime", "updateBy", "password", "salt"])
    fun putUser(@RequestBody data: SysUser): ResponseEntity<*> {
        return ResponseEntity.ok(ResponseDto().apply {
            this.data = userService.modify(data)
        })
    }

    @ApiOperation("注册")
    @PostMapping
    @ZenResponseFilter(type = SysUser::class, include = ["id", "enable", "account", "username", "position", "address", "staffNo", "remark"])
    fun register(@RequestBody entity: SysUser): ResponseEntity<*> {
        return ResponseEntity.ok(ResponseDto().data(userService.register(entity)))
    }

    @DeleteMapping("{id}")
    fun delete(@PathVariable id: Long): ResponseEntity<*> {
        userService.delete(id)
        return ResponseEntity.ok(ResponseDto())
    }

    @ApiOperation("查询用户列表")
    @GetMapping("/list")
    @ZenResponseFilters(
            ZenResponseFilter(type = SysUser::class, filter = ["createTime", "createBy", "updateTime", "updateBy", "password", "salt"]),
            ZenResponseFilter(type = SysRole::class, include = ["id", "name"])
    )
    fun list(@RequestParam(required = false) keyword: String?,
             @RequestParam(required = false) searchOption: Int?,
             @RequestParam(required = false) enable: Boolean?): ResponseEntity<*> {
        val result: Any = if (PaginationUtil.paginable(getRequest())) {
            userService.getUserListPage(keyword, searchOption, PaginationUtil.getRequestPage(getRequest())!!, enable)
        } else {
            userService.getUserList(keyword, searchOption, enable)
        }
        return ResponseEntity.ok(ResponseDto().apply {
            this.data = result
        })
    }


    @GetMapping("/info")
    @ZenResponseFilters(
            ZenResponseFilter(type = SysUser::class, include = ["id", "enable", "account", "username", "position", "address", "staffNo", "remark"]),
            ZenResponseFilter(type = SysRole::class, include = ["id", "enable", "name", "code", "perms", "remark"])
    )
    fun userInfo(@RequestParam platformId: Long): ResponseEntity<*> {
        return ResponseEntity.ok(ResponseDto().apply {
            data = mutableMapOf(
                    "info" to SSOUtil.getCurUserInfo(),
                    "menu" to menuService.getUserMenuTree(platformId),
                    "roles" to SSOUtil.getCurUserRoles(),
                    "perms" to SSOUtil.getCurUserPerms()
            )
        })
    }

    @ApiOperation("获取自身拥有的资源列表")
    @GetMapping("/resource")
    fun getResourceByUserId(): ResponseEntity<*> {
        return ResponseEntity.ok(ResponseDto().apply {
            this.data = SSOUtil.getCurUserPerms()
        })
    }
}